const pws = require("pws");

class Adapter {

    overlay = document.createElement('div');
    popupContainer = document.createElement('div');
    spinnerContainer = document.createElement('div');
    spinner = document.createElement('div');
    iframe = document.createElement('iframe');

    constructor(apiKey, externalId, widthP = 0, heightP = 0, widthA = 0, heightA = 0) {
        this.apiKey = apiKey;
        this.externalId = externalId;
        this.widthP = widthP;
        this.widthA = widthA;
        this.heightP = heightP;
        this.heightA = heightA;
        // this.init();
    }

    init = () => {
        console.log('apiKey: ', this.apiKey);
        console.log('externalId: ', this.externalId);
        const start = Date.now();
        let point = Date.now()

        this.ws = new pws('wss://wss-dev-svkvb.ondigitalocean.app/ws')
        this.ws.onopen = (event) => {
            point = Date.now()
            console.log('connected in ms', (point - start) / 1000)
            this.ws.send(JSON.stringify({
                action: 0,
                userId: this.externalId,
                apiKey: this.apiKey
            }))
        }

        this.ws.onmessage = (message) => {
            point = Date.now()
            console.log('message back in ms', (point - start) / 1000)
            console.log('message::: ', message)

            if (message.data === 'open') this.buildIframe();
        }

        this.setActionButton();

        // channel.bind('my-event', function (data) {

        // });
    }

    setActionButton = () => {
        var styleElement = document.createElement('style');

        styleElement.textContent = `
            .moon-action-button {
              position: fixed;
              bottom: 50px;
              right: 50px;
              background: url('https://cdn.jsdelivr.net/npm/moon-web-worker/resources/trigger_icon.png') center center no-repeat;
              background-size: contain;
              width: 10%;
              height: 10%;
              border: none;
              cursor: pointer;
            }
          `;

        // Append the style element to the head
        document.head.appendChild(styleElement);

        // Create a button element
        const button = document.createElement('button');

        // Add a class to style the button
        button.classList.add('moon-action-button');
        button.onclick = () => this.buildIframe(this.widthP, this.heightP, this.widthA, this.heightA);

        // Append the button to the body
        document.body.appendChild(button);
    }

    chooseValue = (valP, valA, valDef) => valP !== 0 ? `${valP}%` : valA !== 0 ? `${valA}px` : valDef;

    getRightWidthDesktop = () => window.innerWidth > 1920 ? '75%' : '84%';
    getRightHeightDesktop = () => window.innerWidth > 1440 ? '75%' : '84%';

    buildIframe = (widthP, heightP, widthA, heightA) => {
        this.overlay.style.position = 'fixed';
        this.overlay.style.top = '0';
        this.overlay.style.left = '0';
        this.overlay.style.width = '100%';
        this.overlay.style.height = '100%';
        this.overlay.style.backgroundColor = 'rgba(0, 0, 0, 0.5)';
        this.overlay.style.zIndex = '999';

        this.popupContainer.style.position = 'fixed';
        this.popupContainer.style.top = '50%';
        this.popupContainer.style.left = '50%';
        this.popupContainer.style.transform = 'translate(-50%, -50%)';
        this.popupContainer.style.width = this.chooseValue(widthP, widthA, this.isMobile() ? '100%' : this.getRightWidthDesktop());
        this.popupContainer.style.height = this.chooseValue(heightP, heightA, this.isMobile() ? '80%' : this.getRightHeightDesktop());
        this.popupContainer.style.backgroundColor = 'transparent';
        this.popupContainer.style.zIndex = '1000';
        this.popupContainer.style.overflow = 'hidden';

        this.spinnerContainer.style.position = 'absolute';
        this.spinnerContainer.style.top = '50%';
        this.spinnerContainer.style.left = '50%';
        this.spinnerContainer.style.transform = 'translate(-50%, -50%)';

        this.spinner.style.width = '50px';
        this.spinner.style.height = '50px';
        this.spinner.style.border = '5px solid #f3f3f3';
        this.spinner.style.borderTop = '5px solid #3498db';
        this.spinner.style.borderRadius = '50%';
        this.spinner.style.margin = 'auto';
        this.spinner.style.animation = 'spin 1s linear infinite';

        this.spinnerContainer.appendChild(this.spinner);

        this.iframe.src = `https://moon-client.vercel.app/dm?apiKey=${this.apiKey}&externalId=${this.externalId}`; // Set the URL of your iframe content
        this.iframe.style.position = 'fixed';
        this.iframe.style.top = '50%';
        this.iframe.style.left = '50%';
        this.iframe.style.transform = 'translate(-50%, -50%)';
        this.iframe.style.width = '95%';
        this.iframe.style.height = '95%';
        this.iframe.style.border = 'none';
        this.iframe.style.borderRadius = '15px';
        this.iframe.style.display = 'none';

        this.popupContainer.appendChild(this.spinnerContainer);
        this.popupContainer.appendChild(this.iframe);

        document.body.appendChild(this.overlay);
        document.body.appendChild(this.popupContainer);

        // this.iframe.addEventListener('load', function () {
        //     this.showLoadedIframe();
        // });

        const _this = this;
        this.overlay.addEventListener('click', function () {
            _this.removeIframe();
        });
    }

    showLoadedIframe = () => {
        this.spinner.style.display = 'none';
        this.iframe.style.filter = 'drop-shadow(0px 7.5px 3.75px rgba(0, 0, 0, 0.5))';
        this.iframe.style.boxShadow = 'inset -1.5px -1.5px 0px rgba(153, 249, 249, 0.15), inset 1.5px 1.5px 0px rgba(255, 255, 255, 0.15)';
        this.iframe.style.display = 'block';
    }

    removeIframe = () => {
        document.body.removeChild(this.overlay);
        document.body.removeChild(this.popupContainer);
    }

    isMobile = () => /iPhone|iPad|iPod|Android/i.test(navigator.userAgent);

    exports = {
        init: this.init,
        removeIframe: this.removeIframe,
        showLoadedIframe: this.showLoadedIframe
    };
}

module.exports = Adapter;
